<?php
/*----------------------------------------------------------------------------------|  www.giz.de  |----/
	Deutsche Gesellschaft für International Zusammenarbeit (GIZ) Gmb 
/-------------------------------------------------------------------------------------------------------/

	@version		3.4.x
	@build			2nd March, 2022
	@created		15th June, 2012
	@package		Cost Benefit Projection
	@subpackage		view.html.php
	@author			Llewellyn van der Merwe <http://www.vdm.io>	
	@owner			Deutsche Gesellschaft für International Zusammenarbeit (GIZ) Gmb
	@copyright		Copyright (C) 2015. All Rights Reserved
	@license		GNU/GPL Version 2 or later - http://www.gnu.org/licenses/gpl-2.0.html
	
/-------------------------------------------------------------------------------------------------------/
	Cost Benefit Projection Tool.
/------------------------------------------------------------------------------------------------------*/

// No direct access to this file
defined('_JEXEC') or die('Restricted access');

/**
 * Costbenefitprojection View class for the Health_data_sets
 */
class CostbenefitprojectionViewHealth_data_sets extends JViewLegacy
{
	/**
	 * Health_data_sets view display method
	 * @return void
	 */
	function display($tpl = null)
	{
		if ($this->getLayout() !== 'modal')
		{
			// Include helper submenu
			CostbenefitprojectionHelper::addSubmenu('health_data_sets');
		}

		// Assign data to the view
		$this->items = $this->get('Items');
		$this->pagination = $this->get('Pagination');
		$this->state = $this->get('State');
		$this->user = JFactory::getUser();
		// Add the list ordering clause.
		$this->listOrder = $this->escape($this->state->get('list.ordering', 'a.id'));
		$this->listDirn = $this->escape($this->state->get('list.direction', 'DESC'));
		$this->saveOrder = $this->listOrder == 'a.ordering';
		// set the return here value
		$this->return_here = urlencode(base64_encode((string) JUri::getInstance()));
		// get global action permissions
		$this->canDo = CostbenefitprojectionHelper::getActions('health_data');
		$this->canEdit = $this->canDo->get('health_data.edit');
		$this->canState = $this->canDo->get('health_data.edit.state');
		$this->canCreate = $this->canDo->get('health_data.create');
		$this->canDelete = $this->canDo->get('health_data.delete');
		$this->canBatch = $this->canDo->get('core.batch');

		// We don't need toolbar in the modal window.
		if ($this->getLayout() !== 'modal')
		{
			$this->addToolbar();
			$this->sidebar = JHtmlSidebar::render();
			// load the batch html
			if ($this->canCreate && $this->canEdit && $this->canState)
			{
				$this->batchDisplay = JHtmlBatch_::render();
			}
		}
		
		// Check for errors.
		if (count($errors = $this->get('Errors')))
		{
			throw new Exception(implode("\n", $errors), 500);
		}

		// Display the template
		parent::display($tpl);

		// Set the document
		$this->setDocument();
	}

	/**
	 * Setting the toolbar
	 */
	protected function addToolBar()
	{
		JToolBarHelper::title(JText::_('COM_COSTBENEFITPROJECTION_HEALTH_DATA_SETS'), 'health');
		JHtmlSidebar::setAction('index.php?option=com_costbenefitprojection&view=health_data_sets');
		JFormHelper::addFieldPath(JPATH_COMPONENT . '/models/fields');

		if ($this->canCreate)
		{
			JToolBarHelper::addNew('health_data.add');
		}

		// Only load if there are items
		if (CostbenefitprojectionHelper::checkArray($this->items))
		{
			if ($this->canEdit)
			{
				JToolBarHelper::editList('health_data.edit');
			}

			if ($this->canState)
			{
				JToolBarHelper::publishList('health_data_sets.publish');
				JToolBarHelper::unpublishList('health_data_sets.unpublish');
				JToolBarHelper::archiveList('health_data_sets.archive');

				if ($this->canDo->get('core.admin'))
				{
					JToolBarHelper::checkin('health_data_sets.checkin');
				}
			}

			// Add a batch button
			if ($this->canBatch && $this->canCreate && $this->canEdit && $this->canState)
			{
				// Get the toolbar object instance
				$bar = JToolBar::getInstance('toolbar');
				// set the batch button name
				$title = JText::_('JTOOLBAR_BATCH');
				// Instantiate a new JLayoutFile instance and render the batch button
				$layout = new JLayoutFile('joomla.toolbar.batch');
				// add the button to the page
				$dhtml = $layout->render(array('title' => $title));
				$bar->appendButton('Custom', $dhtml, 'batch');
			}

			if ($this->state->get('filter.published') == -2 && ($this->canState && $this->canDelete))
			{
				JToolbarHelper::deleteList('', 'health_data_sets.delete', 'JTOOLBAR_EMPTY_TRASH');
			}
			elseif ($this->canState && $this->canDelete)
			{
				JToolbarHelper::trash('health_data_sets.trash');
			}

			if ($this->canDo->get('core.export') && $this->canDo->get('health_data.export'))
			{
				JToolBarHelper::custom('health_data_sets.exportData', 'download', '', 'COM_COSTBENEFITPROJECTION_EXPORT_DATA', true);
			}
		}
		if ($this->user->authorise('health_data.bulk_export', 'com_costbenefitprojection'))
		{
			// add Bulk Export button.
			JToolBarHelper::custom('health_data_sets.getBulkExport', 'download custom-button-getbulkexport', '', 'COM_COSTBENEFITPROJECTION_BULK_EXPORT', false);
		}

		if ($this->canDo->get('core.import') && $this->canDo->get('health_data.import'))
		{
			JToolBarHelper::custom('health_data_sets.importData', 'upload', '', 'COM_COSTBENEFITPROJECTION_IMPORT_DATA', false);
		}

		// set help url for this view if found
		$this->help_url = CostbenefitprojectionHelper::getHelpUrl('health_data_sets');
		if (CostbenefitprojectionHelper::checkString($this->help_url))
		{
				JToolbarHelper::help('COM_COSTBENEFITPROJECTION_HELP_MANAGER', false, $this->help_url);
		}

		// add the options comp button
		if ($this->canDo->get('core.admin') || $this->canDo->get('core.options'))
		{
			JToolBarHelper::preferences('com_costbenefitprojection');
		}

		// Only load publish filter if state change is allowed
		if ($this->canState)
		{
			JHtmlSidebar::addFilter(
				JText::_('JOPTION_SELECT_PUBLISHED'),
				'filter_published',
				JHtml::_('select.options', JHtml::_('jgrid.publishedOptions'), 'value', 'text', $this->state->get('filter.published'), true)
			);
		}

		JHtmlSidebar::addFilter(
			JText::_('JOPTION_SELECT_ACCESS'),
			'filter_access',
			JHtml::_('select.options', JHtml::_('access.assetgroups'), 'value', 'text', $this->state->get('filter.access'))
		);

		// Set Causerisk Name Selection
		$this->causeriskNameOptions = JFormHelper::loadFieldType('Causesrisks')->options;
		// We do some sanitation for Causerisk Name filter
		if (CostbenefitprojectionHelper::checkArray($this->causeriskNameOptions) &&
			isset($this->causeriskNameOptions[0]->value) &&
			!CostbenefitprojectionHelper::checkString($this->causeriskNameOptions[0]->value))
		{
			unset($this->causeriskNameOptions[0]);
		}
		// Only load Causerisk Name filter if it has values
		if (CostbenefitprojectionHelper::checkArray($this->causeriskNameOptions))
		{
			// Causerisk Name Filter
			JHtmlSidebar::addFilter(
				'- Select ' . JText::_('COM_COSTBENEFITPROJECTION_HEALTH_DATA_CAUSERISK_LABEL') . ' -',
				'filter_causerisk',
				JHtml::_('select.options', $this->causeriskNameOptions, 'value', 'text', $this->state->get('filter.causerisk'))
			);
		}

		// Set Year Selection
		$this->yearOptions = $this->getTheYearSelections();
		// We do some sanitation for Year filter
		if (CostbenefitprojectionHelper::checkArray($this->yearOptions) &&
			isset($this->yearOptions[0]->value) &&
			!CostbenefitprojectionHelper::checkString($this->yearOptions[0]->value))
		{
			unset($this->yearOptions[0]);
		}
		// Only load Year filter if it has values
		if (CostbenefitprojectionHelper::checkArray($this->yearOptions))
		{
			// Year Filter
			JHtmlSidebar::addFilter(
				'- Select '.JText::_('COM_COSTBENEFITPROJECTION_HEALTH_DATA_YEAR_LABEL').' -',
				'filter_year',
				JHtml::_('select.options', $this->yearOptions, 'value', 'text', $this->state->get('filter.year'))
			);
		}

		// Set Country Name Selection
		$this->countryNameOptions = JFormHelper::loadFieldType('Countries')->options;
		// We do some sanitation for Country Name filter
		if (CostbenefitprojectionHelper::checkArray($this->countryNameOptions) &&
			isset($this->countryNameOptions[0]->value) &&
			!CostbenefitprojectionHelper::checkString($this->countryNameOptions[0]->value))
		{
			unset($this->countryNameOptions[0]);
		}
		// Only load Country Name filter if it has values
		if (CostbenefitprojectionHelper::checkArray($this->countryNameOptions))
		{
			// Country Name Filter
			JHtmlSidebar::addFilter(
				'- Select ' . JText::_('COM_COSTBENEFITPROJECTION_HEALTH_DATA_COUNTRY_LABEL') . ' -',
				'filter_country',
				JHtml::_('select.options', $this->countryNameOptions, 'value', 'text', $this->state->get('filter.country'))
			);
		}

		// Only load published batch if state and batch is allowed
		if ($this->canState && $this->canBatch)
		{
			JHtmlBatch_::addListSelection(
				JText::_('COM_COSTBENEFITPROJECTION_KEEP_ORIGINAL_STATE'),
				'batch[published]',
				JHtml::_('select.options', JHtml::_('jgrid.publishedOptions', array('all' => false)), 'value', 'text', '', true)
			);
		}

		// Only load access batch if create, edit and batch is allowed
		if ($this->canBatch && $this->canCreate && $this->canEdit)
		{
			JHtmlBatch_::addListSelection(
				JText::_('COM_COSTBENEFITPROJECTION_KEEP_ORIGINAL_ACCESS'),
				'batch[access]',
				JHtml::_('select.options', JHtml::_('access.assetgroups'), 'value', 'text')
			);
		}

		// Only load Causerisk Name batch if create, edit, and batch is allowed
		if ($this->canBatch && $this->canCreate && $this->canEdit)
		{
			// Causerisk Name Batch Selection
			JHtmlBatch_::addListSelection(
				'- Keep Original '.JText::_('COM_COSTBENEFITPROJECTION_HEALTH_DATA_CAUSERISK_LABEL').' -',
				'batch[causerisk]',
				JHtml::_('select.options', $this->causeriskNameOptions, 'value', 'text')
			);
		}

		// Only load Year batch if create, edit, and batch is allowed
		if ($this->canBatch && $this->canCreate && $this->canEdit)
		{
			// Year Batch Selection
			JHtmlBatch_::addListSelection(
				'- Keep Original '.JText::_('COM_COSTBENEFITPROJECTION_HEALTH_DATA_YEAR_LABEL').' -',
				'batch[year]',
				JHtml::_('select.options', $this->yearOptions, 'value', 'text')
			);
		}

		// Only load Country Name batch if create, edit, and batch is allowed
		if ($this->canBatch && $this->canCreate && $this->canEdit)
		{
			// Country Name Batch Selection
			JHtmlBatch_::addListSelection(
				'- Keep Original '.JText::_('COM_COSTBENEFITPROJECTION_HEALTH_DATA_COUNTRY_LABEL').' -',
				'batch[country]',
				JHtml::_('select.options', $this->countryNameOptions, 'value', 'text')
			);
		}
	}

	/**
	 * Method to set up the document properties
	 *
	 * @return void
	 */
	protected function setDocument()
	{
		if (!isset($this->document))
		{
			$this->document = JFactory::getDocument();
		}
		$this->document->setTitle(JText::_('COM_COSTBENEFITPROJECTION_HEALTH_DATA_SETS'));
		$this->document->addStyleSheet(JURI::root() . "administrator/components/com_costbenefitprojection/assets/css/health_data_sets.css", (CostbenefitprojectionHelper::jVersion()->isCompatible('3.8.0')) ? array('version' => 'auto') : 'text/css');
	}

	/**
	 * Escapes a value for output in a view script.
	 *
	 * @param   mixed  $var  The output to escape.
	 *
	 * @return  mixed  The escaped value.
	 */
	public function escape($var)
	{
		if(strlen($var) > 50)
		{
			// use the helper htmlEscape method instead and shorten the string
			return CostbenefitprojectionHelper::htmlEscape($var, $this->_charset, true);
		}
		// use the helper htmlEscape method instead.
		return CostbenefitprojectionHelper::htmlEscape($var, $this->_charset);
	}

	/**
	 * Returns an array of fields the table can be sorted by
	 *
	 * @return  array  Array containing the field name to sort by as the key and display text as value
	 */
	protected function getSortFields()
	{
		return array(
			'a.ordering' => JText::_('JGRID_HEADING_ORDERING'),
			'a.published' => JText::_('JSTATUS'),
			'g.name' => JText::_('COM_COSTBENEFITPROJECTION_HEALTH_DATA_CAUSERISK_LABEL'),
			'a.year' => JText::_('COM_COSTBENEFITPROJECTION_HEALTH_DATA_YEAR_LABEL'),
			'h.name' => JText::_('COM_COSTBENEFITPROJECTION_HEALTH_DATA_COUNTRY_LABEL'),
			'a.id' => JText::_('JGRID_HEADING_ID')
		);
	}

	protected function getTheYearSelections()
	{
		// Get a db connection.
		$db = JFactory::getDbo();

		// Create a new query object.
		$query = $db->getQuery(true);

		// Select the text.
		$query->select($db->quoteName('year'));
		$query->from($db->quoteName('#__costbenefitprojection_health_data'));
		$query->order($db->quoteName('year') . ' ASC');

		// Reset the query using our newly populated query object.
		$db->setQuery($query);

		$results = $db->loadColumn();
		$_filter = array();

		if ($results)
		{
			// get model
			$model = $this->getModel();
			$results = array_unique($results);
			foreach ($results as $year)
			{
				// Translate the year selection
				$text = $model->selectionTranslation($year,'year');
				// Now add the year and its text to the options array
				$_filter[] = JHtml::_('select.option', $year, JText::_($text));
			}
		}
		return $_filter;
	}
}
